function nmc = balanceInhExc(nmc, p, Kfnc)
% balanceInhExc Balance inhibition and excitation
%
% Syntax
%
%    nmc = balanceInhExc(nmc, p, Kfnc);
%
%  Description
%  
%    nmc = balanceInhExc(nmc, p, Kfnc) scales the weights of the
%      inhibitory synapses onto a neuron in pool p such that is
%      counterbalances the excitatory input. Kfnc is a cell array
%      which specifies a distribution to draw the ratio INH/EXC from
%      (default: { @unirnd, 0.99, 1.0 }). Kfnc{1} is a function handle
%      to a random nuber generating function K{2:n} are parameters for
%      this function. The function specified by Kfnc{1} must also
%      accept the parameters m and n as last arguments to specify the
%      size of the matrix of random numbers to generate.
%
%  Algorithm
%
%    read the code
%
%  See also
%
%    balanceExcThresh
%
  
  
  if nargin < 2, p=1; end
  if nargin < 3, Kfnc= { @unirnd , 0.9, 1.1 }; end
  
  global_definitions

  pool = get(nmc, 'pool');
  postIdx  = [pool(p).neuronIdx];
  nIdx = length(postIdx);
  
  if iscell(Kfnc)
    K=feval(Kfnc{:},1,nIdx);
  elseif isnumeric(Kfnc)
    K=Kfnc*ones(1,nIdx);
  end

  for i=1:nIdx
    
    [inSyn,outSyn]=csim('get',postIdx(i),'connections');

    % find presynaptic neurons
    preIdx = zeros(1,length(inSyn));
    for s=1:length(inSyn)
      [preIdx(s),dummy]=csim('get',inSyn(s),'connections');
      preType(s)=csim('get',preIdx(s),'type');
      w(s)=csim('get',inSyn(s),'W');
      tau_s(s)=csim('get',inSyn(s),'tau');
    end

    % get type of neurons
    inhIdx = find(preType==INH);
    excIdx = find(preType==EXC);

    % calculate synaptic efficacy
    eff=w.*tau_s;
    
    % total excitatory input
    effE=abs(sum(eff(excIdx)));
    
    % total inhibitory input
    effI=abs(sum(eff(inhIdx)));


    fprintf('% 3i: I/E : %i / %i (%i) :  %g / %g = %g\n',i,length(inhIdx),length(excIdx),length(inSyn),effE*K(i),effI,K(i));
    
    % adjust inhibition
    w(inhIdx) = K(i)*effE*w(inhIdx)/effI;
    
    for s=1:length(inSyn)
      csim('set',inSyn(s),'W',w(s));
    end
    
  end
  