function [O,Y1] = apply(this,X)
% APPLY - Perceptron forward propagation
%
%   Y = APPLY(VPERC, X)
%   Compute output Y of the Perceptron given by VPERC.NET for inputs X.
%   X must contain one input vector per row. Y is a column vector with
%   one entry for each input vector in X. Y(i) is the Perceptron output
%   for input vector X(i,:).
%   [Y, Y1] = APPLY(VPERC, X) also returns the matrix Y1, the output of
%   each of the VPERC.NET.VOTES perceptrons before applying the activation
%   function and voting.
%   Different voting schemes may be used to combine perceptron
%   outputs. The voting scheme is given by NET.VOTING, valid schemes are
%     'majority': The +1/-1 prediction of individual Perceptrons is
%       computed, the overall prediction is given by the majority of the
%       individual Perceptrons.
%     'avgweight': The activations of all Perceptrons are summed up, the
%       overall prediction is the sign of this sum. This is equivalent to
%       averaging the weight vectors of individual perceptrons.
%

%
% Copyright (c) by Anton Schwaighofer (2001)
% $Revision: 1.1 $ $Date: 2001/12/19 10:38:44 $
% mailto:anton.schwaighofer@gmx.net
%
% Implemented by Stefan Haeusler 2003/05/09 (haeusler@igi.tu-graz.ac.at)

net = this.net;

voting = net.voting;

[N, d] = size(X);
if isempty(net.kernel),
  X(:,net.nin+1) = ones([N 1]);
  Y1 = X*net.weight';
else
  error('Kernel perceptron not yet implemented');
end

switch voting
  case 'majority'
    % Apply activation function, predict the most frequent label
    O = sign(sum(sign(Y1), 2));
  case 'avgweight'
    % Sum perceptron output before activation function (equivalent to:
    % use averaged weight vector), apply activation function
    O = sign(sum(Y1, 2));
  otherwise
    error(sprintf('Unknown voting scheme ''%s''', voting));
end

