function this = train(this,TrainSet,varargin)
%
% Note: Classifier for two classes.
%
% Each element of TrainSet.Y that is >=0
% is treated as class +1, each element <0 is treated as class -1.
%
% Copyright (c) by Anton Schwaighofer (2001)
% $Revision: 1.4 $ $Date: 2002/01/25 09:33:10 $
% mailto:anton.schwaighofer@gmx.net
%
% Implemented by Stefan Haeusler 2003/05/09 (haeusler@igi.tu-graz.ac.at)


[TrainSet,ValidSet] = check_data(this,TrainSet,varargin);

t0=clock;

% create network

if prod(size(this.margin))~=2,
  error('Input parameter MARGIN must be a vector of length 2');
end

net.type = 'perc';
net.nin = size(TrainSet.X,2);
net.nout = 1;

if isempty(this.kernel),
  % linear perceptron: weight vector operates in input space plus bias term
  net.weight = zeros([this.votes net.nin+1]);
else
  % kernel perceptron: use weights on examples
  net.weight = [];
end
net.kernel = this.kernel;
net.margin = this.margin;
net.learnrate = this.learnrate;
net.maxepoch = this.maxepoch;
net.votes = this.votes;
net.verbosity = this.verbosity;
net.voting = this.voting;

% train the network
%
% Each element of Y that is >=0
% is treated as class +1, each element <0 is treated as class -1.
%

X = TrainSet.X;
Y = TrainSet.Y;

[N, d] = size(X);
Y = sign(Y-mean(Y));
margin = net.margin(2-(Y>0));
if isempty(net.kernel),
  % Add dummy column for bias
  X(:,net.nin+1) = ones([N 1]);
  % Pre-compute X(i,:)*Y(i). Store it in transposed order, so that one
  % examples are stored columnwise. This makes computation with sparse X
  % some orders of magnitude faster
  XY = X';
  % Avoid using repmat here
  for i = 1:N,
    XY(:,i) = XY(:,i)*Y(i);
  end
  % Linear perceptron: work in primal (weight) space
  for v = 1:net.votes,
    if net.verbosity>0, fprintf('Training for vote %i ', v); end
    w = net.weight(v,:);
    % updates==0 as long as no errors are found in the current epoch
    updates = 1;
    % current epoch
    epoch = 1;
    % training for each vote in the voted perceptron is based on
    % different permutation of the data, giving different hypotheses
    r = randperm(N);
    % Limit training to net.maxepoch epochs, emergency exit for
    % non-separable data sets
    while (epoch<net.maxepoch) & (updates>0)
      if all(w==0),
        % Initialise with first example, make sure this also works for
        % sparse inputs. No 'intelligent' initialisation here, we wish to
        % obtain different hypotheses for each vote!
        w = double(XY(:,r(1))');
      end
      if net.verbosity>1, fprintf('.'); end
      updates = 0;
      for i = r,
        % Prediction with current hypothesis, multiplied by target
        yPred = w*XY(:,i);
        % Check whether margin constraint is violated. Outputs must be
        % yPred>=margin for positive examples, yPred<=margin for negative examples
        if yPred <= margin(i),
          % update weight vector
          w = w+net.learnrate*XY(:,i)';
          updates = updates+1;
        end
      end
      if net.verbosity>1, fprintf('%i ', updates); end
      epoch = epoch+1;
    end
    if net.verbosity>0, fprintf('\n'); end
    net.weight(v,:) = w;
  end
else
  error('Kernel perceptron not yet implemented');
end

this.net = net;

this.time=etime(clock,t0);
