%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulate various cortical microcircuit models to plot spike rasters
%
%   or 
%
% Train readout neurons of various cortical microcircuit models
% to perform regression and classification tasks
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all

%%%%%%%%%%%%%%%%%%%%%%%%%
% CSIM STARTUP
%%%%%%%%%%%%%%%%%%%%%%%%%

LSMROOT='../..';  % set LSMROOT to the root directory

addpath(LSMROOT);
lsm_startup;


%%%%%%%%%%%%%%%%%%%%%%%%%
% GENERAL PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%

%  PURPOSE = 0; % train circuits on a specific task
PURPOSE = 1; % simulate circuit twice and output spike rasters and firing statistics as in Figure 2


global VERBOSE_LEVEL;           
global PLOTTING_LEVEL;          

VERBOSE_LEVEL  = 2;           % the higher the more is printed at stdout 
PLOTTING_LEVEL = 0;           % the higher the more plots will appear

% parameters to save results
SAVE = 0;	% set to 1 if results should be saved
datapath = './data1'; % path where files are saved
FILEIDX = 1	% start file index contained in the file name

% set random seeds so that different matlab session generate different circuits
rand('state',sum(100*clock))
randn('state',sum(100*clock))

%%%%%%%%%%%%%%%%%%%%%%%%
% INPUT DISTRIBUTION
%%%%%%%%%%%%%%%%%%%%%%%%

Tmax = 0.45;            % length of stimulus
Tstim = 0.03;           % length of input segment
dt_bin = 1e-3;          % bining calculate the actual input rates 

jitter = [1e-3];	% SD of Gaussian jitter added to the input spike trains


%%%%%%%%%%%%%%%%%%%%%%%%
% CIRCUIT PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%

noise = 1;	% activate noise
Cscales = 1;	% general scaling parameter for connection strengths

WINscales = [0.0450  0.0450  0.0450  0.1106 ]'; % scaling parameter for input weights 
Wscales = 20.2500;	% scaling parameter for recurrent weights

%%%%%%%%%%%%%%%%%%%%%%%%
% LEARNING SETUP
%%%%%%%%%%%%%%%%%%%%%%%%

% select purpose of the simulation
if (PURPOSE)
   % simulate circuit twice and output spike rasters and firing statistics as in Figure 2
   num_train = 1        % (works not with regression eval function)
   num_test = 1

   nTrials = 1;         % number of total trials
else
   % train circuits on a specific task (circuit type and task defined below)
   num_train = 1500;    % number of training samples
   num_test = 300;      % number of test samples

   nTrials = 20;        % number of total trials
end

% define task

%TASK = 'class'
  TASK = 'regress'

if strcmp(TASK,'class')
   eval_fct = 'eval_class';         % function name to evaluate performance
   target_fct = 'target_class';     % function name to generate training and test targets 

   nInputChannels = 40;             %number of input channels
   ntcl_idx = 12:15;                % time segments used for analysis

   output_binwidth = [15e-3];       % convolution time constant for output spike trains
   input_binwidth = [15e-3];        % placeholder, only used for regression tasks

   freq = [20 20];                  % frequencies for poisson inputs of both classes

elseif strcmp(TASK,'regress')
   eval_fct = 'eval_rate';          % function name to evaluate performance
   target_fct = 'target_rate';      % function name to generate training and test targets 

   nInputChannels = 4;              % number of input channels
   ntcl_idx = round(Tmax/Tstim);    % time segments used for analysis

   output_binwidth = [15e-3];       % convolution time constant for output spike trains
   input_binwidth = [15e-3];        % bin width to calculate input mean firing rate

   % constant rate templates 
 
   nRates1 = Inf;      % infinite number of mean firing rate values in [fmin1,fmax1]
   nRates2 = Inf;      % infinite number of mean firing rate values in [fmin2,fmax2]

   fmax1 = 25;         % maximum mean firing rate of class 1 Poisson inputs
   fmax2 = 25;         % maximum mean firing rate of class 2 Poisson inputs
   fmin1 = 15;         % minimum mean firing rate of class 1 Poisson inputs
   fmin2 = 15;         % minimum mean firing rate of class 2 Poisson inputs

else
   error('Task not defined!')
end


%%%%%%%%%%%%%%%%%%%%%%%%
% CIRCUIT parameters
%%%%%%%%%%%%%%%%%%%%%%%%

% select circuit type

NETWORK_TYPE = 'V1'                 % data-based circuits 
% NETWORK_TYPE = 'amorphous'        % amorphous circuits
% NETWORK_TYPE = 'smallworld'       % small world circuits
% NETWORK_TYPE = 'fanin'            % degree controlled circuit 
% NETWORK_TYPE = 'fanin_nolayer'    % degree controlled circuitw/o layer specificity

NEURON_TYPE = 'HH';                 % Hodgkin-Huxley type neurons

Wset = {0,0,0}; % each true element activates a specific 
                % connectivity correction (none is used for the paper)

layer_base  = [7 8]; % define size of the circuit = layer_base * 10 [neurons]

PERM_SYNDYN = 0; % 1: permute synaptic dynamics parameters U,D,F 

warning off

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GENERATE SAVE FILE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% write file to show that process started

filename = sprintf('%s/data_%s_%s_%s_W%i%i%i_n%i_%i.mat',datapath,TASK,NETWORK_TYPE,NEURON_TYPE,Wset{:},noise,FILEIDX);
while ~isempty(dir(filename))
   FILEIDX = FILEIDX + 1;
   filename = sprintf('%s/data_%s_%s_%s_W%i%i%i_n%i_%i.mat',datapath,TASK,NETWORK_TYPE,NEURON_TYPE,Wset{:},noise,FILEIDX);
end
if (SAVE)
   save(filename,'FILEIDX')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EXHAUSTIVE PARAMETERS SEARCH SETUP (not used below)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% set scaling parameters for the synaptic weights
WIN1start =  WINscales(1);
WIN2start =  WINscales(4);
WRstart   =  Wscales;

for trial=1:nTrials;

  % set default weights

  WINscales(1:3) = 1.1 * WIN1start;
  WINscales(4) = 1.1 * WIN2start;
  Wscales = 1.1 * WRstart;

  % define input class for the selected task 
  define_input

  % loop over different synpatic scaling parameter settings

  run_idx = 0;
  for l=1:length(Cscales)
    Cscale = Cscales(l);
    for win = 1:size(WINscales,2)
     WINscale = WINscales(:,win);
     for w = 1:length(Wscales)
       Wscale = Wscales(w);
 
       % generate network

       network_name = sprintf('make_network_%s_%s',NETWORK_TYPE,NEURON_TYPE);
       if VERBOSE_LEVEL
          disp(network_name)
       end 
       eval(sprintf('PCT = %s(Cscale,Wscale,WINscale,nInputChannels,layer_base,Wset,noise);',network_name))
       nmc = generate(PCT);

       if (PERM_SYNDYN)
          disp('ATTENTION: Synaptic dynamics parameters U,D,F permuted!')
          perm_synUDF
       end

       run_idx = run_idx + 1;
       fprintf('*** Trial %d (run %d): Cscale %g, WINscale [%g,%g,%g,%g], Wscale %g,  ***\n',trial,run_idx, Cscale, WINscale, Wscale);

       % store network parameters
       simres(trial).run(run_idx).net.NETWORK_TYPE = NETWORK_TYPE;
       simres(trial).run(run_idx).net.NEURON_TYPE = NEURON_TYPE;
       simres(trial).run(run_idx).net.Wset = Wset;
       simres(trial).run(run_idx).net.Wscale = Wscale;
       simres(trial).run(run_idx).net.WINscale = WINscale;
       simres(trial).run(run_idx).net.Tscale = 1; % default value for this field
       simres(trial).run(run_idx).net.numsynInp = NaN;
       simres(trial).run(run_idx).net.numsynNet = NaN;

       % simulate for training samples
       [train_states,I1tr,I2tr,nN23f,nN5f,IdxE,IdxI] = collect_all_data_2input_2output_l25(nmc,output_binwidth,InputDist1,InputDist2,num_train,Tmax);

       % simulate for test samples
       [test_states,I1te,I2te] = collect_all_data_2input_2output_l25(nmc,output_binwidth,InputDist1,InputDist2,num_test,Tmax,nN23f,nN5f,IdxE,IdxI);

       % save stuff into file before learning
       if (SAVE) 
          save(filename,'simres','nmc','InputDist1','InputDist2');
       end

       % set some indices for learning area
       tPoints = [1]; 
       templates = ceil(2/3*Tmax/Tstim):length(2);

       % train network and evaluate test performance
       analyse_dual_temporal_task 

       % analyse firing statistics of different layers
       analyse_layer_statistic

       % save stuff into file after learning
       if (SAVE) 
          save(filename,'simres','nmc','InputDist1','InputDist2');
       end
      end
    end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%
% Results
%%%%%%%%%%%%%%%%%%%%%%%%%

% all results are stored in the variable simres
%
% See 
%
%    simres(trial).run(1).stats(ntcl,1,nReg,nReadout).TrError2
%    simres(trial).run(1).stats(ntcl,1,nReg,nReadout).TeError2
%
% with
%
%    trial = [1:20]   ... trial index
%    ntcl = 1         ... results for time segment t = [-120,90] ms
%    ntcl = 2         ... results for time segment t = [-90,-60] ms
%    ntcl = 3         ... results for time segment t = [-60,-30] ms
%    ntcl = 4         ... results for time segment t = [-30, 00] ms
%
%    nReg             ... task index, see targe_rate.m and target_class.m for task definitions
%
%                         For classification tasks:
%
%                              1   ... classify input 1 template label
%                              2   ... classify input 2 template label
%                              3   ... xor of template labels of input 1 & 2
%
%                         For regression tasks:
%
%                              6   ... r1/r2
%                              7   ... (r1 - r2)^2
%                              13  ... nonlinear component of r1/r2
%                              14  ... nonlinear component of (r1 - r2)^2
%
%    nReadout = 2     ... results for L2/3 readout
%    nReadout = 3     ... results for L5 readout
%
% for train and test results for sign constrained linear regression